import sys
import os.path
import random
import math

import cocos
from cocos.director import director
from cocos.layer import Layer, ColorLayer, MultiplexLayer
from cocos.scene import Scene
from cocos.scenes.transitions import *
from cocos.sprite import Sprite
from cocos.menu import Menu, MenuItem, ToggleMenuItem, MultipleMenuItem, shake, shake_back, zoom_in, zoom_out
from cocos.text import Label
from cocos.actions.interval_actions import *
from cocos.actions.instant_actions import *
from cocos.actions.interval_actions import Reverse, Repeat

import pyglet
from pyglet import gl, font
from pyglet.window import key

import soundex
from constants import *

# -----------------------------------------------------------------------------
big_alpha_codes = [ 163208757248, 240518168576, 231928233984, 171798691840,
                    111669149696, 176093659136, 180388626432, 184683593728,
                    133143986176, 188978561024, 193273528320, 197568495616,
                    249108103168, 244813135872, 137438953472, 141733920768,
                    103079215104, 115964116992, 167503724544, 120259084288,
                    128849018880, 236223201280, 107374182400, 227633266688, 
                    124554051584, 223338299392 ]

box_size = (32, 48)

enemy_names = [w.strip() for w in """enemy-anjuta.png
    enemy-devcpp.png
    enemy-drwatson.png
    enemy-eclipse.png
    enemy-gdb.png
    enemy-kdevelop.png
    enemy-monodev.png
    enemy-netbeans.png
    enemy-qt.png
    enemy-vc8.png
    enemy-windbg.png
    enemy-xcode.png""".splitlines()]

def set_background(layer, level):
    top = (3*HEIGHT)/2
    bottom = -HEIGHT/2
    bkg1 = Sprite("level" + str(level) + "-bkg-1.png", position=(WIDTH/2,top), opacity=128)
    bkg2 = Sprite("level" + str(level) + "-bkg-2.png", position=(WIDTH/2,HEIGHT/2), opacity=128)

    bkg1.do( Repeat(MoveTo((WIDTH/2,bottom), 10) + MoveTo((WIDTH/2,top),0)) )
    bkg2.do( Repeat(MoveTo((WIDTH/2,bottom), 5) +
        MoveTo((WIDTH/2,top), 0) + MoveTo((WIDTH/2,HEIGHT/2), 5) ) )

    layer.add(bkg1, z=0.1)
    layer.add(bkg2, z=0.1)

class StackTrace(cocos.layer.Layer):
    def __init__(self, function_names):
        super(StackTrace, self).__init__()
        self.stack_frame_labels = []

        for function_ix, function_name in enumerate(function_names):
            label = Label(function_name,
                x=780,
                y=15 + 20 * function_ix,
                font_name="Courier", font_size=10,
                anchor_y='center', anchor_x='left',
                color=(64, 64, 64, 255), bold=False)
            self.stack_frame_labels.append(label)
            self.add(label)

        self.arrow = Sprite("arrow-right.png", position=(760, 15 + 20 * (len(self.stack_frame_labels) - 1)),
            color=(255, 255, 0), scale=0.35)
        self.add(self.arrow)
        self._on_stack_frame_activate()

    def unwind(self, layer):
        soundex.play('key-typed.wav')
        soundex.play('phrase-entered.wav')
        if self.stack_frame_labels:
            self.arrow.position = (self.arrow.position[0], self.arrow.position[1] - 20)
            self.remove(self.stack_frame_labels[-1])
            del self.stack_frame_labels[-1]
            if not self.stack_frame_labels:
                self.remove(self.arrow)
                layer.on_level_completed()
            else:
                self._on_stack_frame_activate()

    def _on_stack_frame_activate(self):
        if self.stack_frame_labels:
            self.stack_frame_labels[-1].element.bold = True
            self.stack_frame_labels[-1].element.color = (0, 0, 0, 255)

class Level1():
    def __init__(self):
        self.index = 0
        self.enemy_count = 1
        self.function_names = ["(?)()", "handleTransactionResponse()",
            "createCallback()", "onRead()", "loadRecords()",
            "fireEvent()", "apply()", "detailedComparisonRead()",
            "each()", "push()", "onInvalidEntry()"]
        self.words = [w.strip() for w in """
            void main int float before after get set less raise throw
            except for if var while return ret go jump gosub foreach each
            else elif done do nbsp space alt ctrl enter read write open save
            load test draw update extjs java cpp header tail append prepend
            push pop stack die log exit erase paint hello world print printf
            println null none nil poke peek basic pascal row set dict db select
            remove delete new array struct class pickle python pypy eval calc
            kill killall cd cp ls grep bash sed awk find close create com net
            task thread process file resume suspend match""".split()]

class Level2():
    def __init__(self):
        self.index = 1
        self.enemy_count = 2
        self.function_names = ["run_game", "main.main()", "director.run()"
            "__init__.run()", "win32.run()", "win32._time_func()", "__init__.idle()",
            "clock.tick()", "clock._tick()", "battle_scene._step()", "battle_scene.update()",
            "battle_scene.get_sprite_distance()", "???()"]
        self.words = [w.strip() for w in """
            void main int float before after get set less raise throw
            except for if var while return ret go jump gosub foreach each
            else elif done do nbsp space alt ctrl enter read write open save
            load test draw update extjs java cpp header tail append prepend
            push pop stack die log exit erase paint hello world print printf
            println null none nil poke peek basic pascal row set dict db select
            remove delete new array struct class pickle python pypy eval calc
            kill killall cd cp ls grep bash sed awk find close create com net
            task thread process file resume suspend match""".split()]

class Level3():
    def __init__(self):
        self.index = 2
        self.enemy_count = 3
        self.function_names = ["???", "Application::Run()",
            "Application::ProcessMessages()", "Application::OnIdle()",
            "CommandProcessor::Dispatch()", "boost::signal2::event::_signal()"
            "boost::bind<!@#$%>", "DownloadManager::OnFileDownloaded()",
            "GuiManager::NotifyBaloon()", "ErrorHandler::HandleIconNotFound()"]
        self.words = [w.strip() for w in """
            void main int float before after get set less raise throw
            except for if var while return ret go jump gosub foreach each
            else elif done do nbsp space alt ctrl enter read write open save
            load test draw update extjs java cpp header tail append prepend
            push pop stack die log exit erase paint hello world print printf
            println null none nil poke peek basic pascal row set dict db select
            remove delete new array struct class pickle python pypy eval calc
            kill killall cd cp ls grep bash sed awk find close create com net
            task thread process file resume suspend match""".split()]

class Level4():
    def generate_stack_frame_name(self):
        cs = ""
        for c in xrange(2 * 4):
            cs += random.choice('0123456789abcdef')
        return cs
    
    def __init__(self):
        self.index = 3
        self.enemy_count = 4
        self.function_names = [self.generate_stack_frame_name() for i in xrange(12)]
        self.words = [w.strip() for w in """
            void main int float before after get set less raise throw
            except for if var while return ret go jump gosub foreach each
            else elif done do nbsp space alt ctrl enter read write open save
            load test draw update extjs java cpp header tail append prepend
            push pop stack die log exit erase paint hello world print printf
            println null none nil poke peek basic pascal row set dict db select
            remove delete new array struct class pickle python pypy eval calc
            kill killall cd cp ls grep bash sed awk find close create com net
            task thread process file resume suspend match""".split()]

levels = [Level1(), Level2(), Level3(), Level4()]

# -----------------------------------------------------------------------------

def add_vectors(v1, v2):
    return (v1[0] + v2[0], v1[1] + v2[1])

def sub_vectors(v1, v2):
    return (v1[0] - v2[0], v1[1] - v2[1])

def add_vector_multiplied(v1, m, v2):
    return (v1[0] + m * v2[0], v1[1] + m * v2[1])

def get_vector_length(v):
    return math.sqrt(v[0] * v[0] + v[1] * v[1])

def get_sprite_distance(s1, s2):
    return get_vector_length(sub_vectors(s1.position, s2.position))

class BattleLayer(cocos.layer.ColorLayer):
    is_event_handler = True

    def __init__(self, level_index=3):
        super(BattleLayer, self).__init__(255, 255, 255, 255)
        print "BattleLayer.__init__()"

        self.batch = pyglet.graphics.Batch()

        self.level = levels[level_index]

        # Spawn the avatar.
        self.avatar = Sprite("exception-avatar.png", position=(WIDTH/2,HEIGHT/2))
        self.avatar.velocity=(0, 0)
        self.avatar.angular_velocity = 1
        self.add(self.avatar, z=120)
        self.killed = False

        # Some texts.
        self.text1 = Label("CAUGHT!",
                x=WIDTH/2,
                y=HEIGHT/2 + 200,
                font_name="Arial", font_size=120,
                anchor_y='center', anchor_x='center',
                color=(128, 0, 0, 255), bold=True)
        self.text2 = Label("Type 'raise' or 'throw' to rethrow the hero",
                x=WIDTH/2,
                y=HEIGHT/2 + 70,
                font_name="Arial", font_size=35,
                anchor_y='center', anchor_x='center',
                color=(0, 128, 0, 255), bold=True)

        # Background.
        set_background(self, self.level.index + 1)

        # Stack trace.
        self.stack_trace = StackTrace(self.level.function_names)
        self.add(self.stack_trace, z=10)

        # Central entry label.
        self.entry_label = Label("",
                x=WIDTH/2,
                y=HEIGHT/2,
                font_name="Courier", font_size=32,
                anchor_y='center', anchor_x='center',
                color=(228, 0, 0, 255), bold=True)

        self.label_shadows = []
        self.add(self.entry_label, z=1000)
        self._create_new_entry_label_shadow()

        # Spawn steering labels.
            # 0 - Bottom
        self.phrases = []
        self.phrases.append(Label(self._pick_phrase(),
                x=WIDTH/2,
                y=0,
                font_name="Courier", font_size=32,
                anchor_y='bottom', anchor_x='center',
                color=(0, 0, 228, 255), bold=True))
        self.phrases[-1].transform_anchor = (WIDTH/2, 0)
            # 1 - Left
        self.phrases.append(Label(self._pick_phrase(),
                x=0,
                y=HEIGHT/2,
                font_name="Courier", font_size=32,
                anchor_y='top', anchor_x='center',
                color=(0, 0, 228, 255), bold=True))
        self.phrases[-1].transform_anchor = (0, HEIGHT/2)
        self.phrases[-1].rotation = -90
            # 2 - Right
        self.phrases.append(Label(self._pick_phrase(),
                x=WIDTH,
                y=HEIGHT/2,
                font_name="Courier", font_size=32,
                anchor_y='top', anchor_x='center',
                color=(0, 0, 228, 255), bold=True))
        self.phrases[-1].transform_anchor = (WIDTH, HEIGHT/2)
        self.phrases[-1].rotation = 90
            # 3 - Top
        self.phrases.append(Label(self._pick_phrase(),
                x=WIDTH/2,
                y=HEIGHT,
                font_name="Courier", font_size=32,
                anchor_y='top', anchor_x='center',
                color=(0, 0, 228, 255), bold=True))
        self.phrases[-1].transform_anchor = (WIDTH/2, HEIGHT)
        for p in self.phrases:
            self.add(p, z=60)

        # Spawn the RET.
        self.ret = Sprite("ret.png", scale=0.5)
        self.ret.velocity = (0, 0)
        self.ret.angular_velocity = 0
        self._randomize_ret_position()
        self.add(self.ret, z=50)

        # Spawn the enemies.
        self.enemies = []

        for i in xrange(self.level.enemy_count):
            enemy = Sprite(random.choice(enemy_names),
                position=(random.randint(0, WIDTH), random.randint(0, HEIGHT)))
            enemy.velocity = (random.randint(-30, 30), random.randint(-30, 30))
            enemy.angular_velocity = 0
            enemy.charge = random.uniform(0.3, 0.6)
            self.add(enemy, z=50)
            self.enemies.append(enemy)

    def on_level_completed(self):
        print "The level has been completed."
        for a in self.get_children():
            self.remove(a)
        self.remove_all_handlers()
        director.pop()

    def _randomize_ret_position(self):
        self.ret.position = (random.randint(0, WIDTH), random.randint(0, HEIGHT))
        self.ret.velocity = (0, 0)

    def _pick_phrase(self):
        return random.choice(self.level.words)

    def _create_new_entry_label_shadow(self):
        self.entry_label_shadow = Label("",
                x=WIDTH/2,
                y=HEIGHT/2,
                font_name="Courier", font_size=33,
                anchor_y='center', anchor_x='center',
                color=(197, 0, 0, 128), bold=True)
        self.entry_label_shadow.transform_anchor = (WIDTH/2, HEIGHT/2)
        self.add(self.entry_label_shadow, z=102)
        self.label_shadows.append(self.entry_label_shadow)

    def on_enter(self):
        super(BattleLayer, self).on_enter()
        print "BattleLayer.on_enter()"

    def on_exit(self):
        super(BattleLayer, self).on_exit()
        print "BattleLayer.on_exit()"

    def draw(self):
        super(BattleLayer, self).draw()
        if self.batch:
            self.batch.draw()

    def on_enemy_bite(self):
        self.killed = True
        self.avatar.opacity = 128
        print "The enemy bites you!"
        self.add(self.text1, z=300)
        self.add(self.text2, z=300)
        soundex.play('caught.wav')

    def back_to_live(self):
        print "The Exception has been rethrown!"
        self.killed = False
        self.avatar.scale = 1
        self.avatar.opacity = 255
        self.avatar.position = (WIDTH/2, HEIGHT/2)
        self.avatar.velocity=(0, 0)
        self.avatar.angular_velocity = 1
        self.remove(self.text1)
        self.remove(self.text2)

    def update(self, dt):
        if not self.killed:
            self._manage_velocity(self.avatar, dt)
            self._manage_bouncing(self.avatar)

        self.entry_label_shadow.element.text = self.entry_label.element.text
        for label_shadow in self.label_shadows:
            if label_shadow == self.entry_label_shadow:
                continue
            if label_shadow.opacity <= 0:
                self.remove(label_shadow)
                self.label_shadows.remove(label_shadow)
            else:
                label_shadow.opacity -= 4
                label_shadow.scale *= 1.1
                label_shadow.rotation += 4
                label_shadow.position = add_vectors(
                    label_shadow.position,
                    (random.randint(-4, 4), random.randint(-4, 4)))

        self.entry_label_shadow.position = add_vectors(
            self.entry_label.position,
            (random.randint(-4, 4), random.randint(-4, 4)))

        # Manage RET.
        d = get_sprite_distance(self.avatar, self.ret)
        if not self.killed:
            self._manage_attraction(self.ret, self.avatar, dt, 0.1)
            self._manage_attraction(self.avatar, self.ret, dt, 0.1)
        self._manage_velocity(self.ret, dt)
        self._manage_bouncing(self.ret)

        if not self.killed:
            # Check for avatar-RET collision.
            if d < 60:
                self.stack_trace.unwind(self)
                self._randomize_ret_position()
                self.avatar.velocity = (0, 0)

        # Manage enemies.
        for e1 in self.enemies:
           for e2 in self.enemies:
               self._manage_attraction(e1, e2, dt, e1.charge * e2.charge)
        for e in self.enemies:
            self._manage_velocity(e, dt)
            self._manage_bouncing(e)
            if not self.killed:
                d = get_sprite_distance(self.avatar, e)
                if d < 40:
                    self.on_enemy_bite()

        if self.killed:
            if self.avatar.scale < 300:
                self.avatar.scale *= 1.1
        pass

    def _bounce_sprite(self, s, boundary_ix):
        soundex.play('bounce.wav')

        if boundary_ix == 0:
            # Bottom.
            s.velocity = (s.velocity[0], math.fabs(s.velocity[1]))
            s.angular_velocity = -s.angular_velocity
        elif boundary_ix == 1:
            # Left.
            s.velocity = (math.fabs(s.velocity[0]), s.velocity[1])
            s.angular_velocity = -s.angular_velocity
        elif boundary_ix == 2:
            # Right.
            s.velocity = (-math.fabs(s.velocity[0]), s.velocity[1])
            s.angular_velocity = -s.angular_velocity
        else:
            # Top.
            s.velocity = (s.velocity[0], -math.fabs(s.velocity[1]))
            s.angular_velocity = -s.angular_velocity

    def _manage_velocity(self, sprite, dt):
        sprite.position = add_vector_multiplied(sprite.position, dt, sprite.velocity)
        sprite.angular_velocity = math.copysign(get_vector_length(sprite.velocity), sprite.angular_velocity)
        sprite.rotation += sprite.angular_velocity * dt

    def _manage_attraction(self, s1, s2, dt, factor):
        d = get_sprite_distance(s1, s2)
        if d < 0.0001:
            d = 0.0001
        d2 = factor / math.sqrt(d * d)
        s1.velocity = add_vector_multiplied(s1.velocity, d2, sub_vectors(s2.position, s1.position))
        s1.angular_velocity = math.copysign(get_vector_length(s1.velocity), s1.angular_velocity)

    def _manage_bouncing(self, s):
        if s.position[0] < 0:
            self._bounce_sprite(s, 1)
        if s.position[0] > WIDTH:
            self._bounce_sprite(s, 2)
        if s.position[1] < 0:
            self._bounce_sprite(s, 0)
        if s.position[1] > HEIGHT:
            self._bounce_sprite(s, 3)

    def on_key_press(self, key_code, modifiers):
        # Recognize the pressed key.
        key_symbol = key.symbol_string(key_code)
        modifiers_string = key.modifiers_string(modifiers)

        in_array = ["MINUS", "EQUAL", "SEMICOLON", "APOSTROPHE",
            "BRACKETLEFT", "BRACKETRIGHT", "BACKSLASH", "COMMA", "PERIOD",
            "SLASH"]
        in_array2 = ["UNDERSCORE", "PLUS", "COLON", "DOUBLEQUOTE", "BRACELEFT",
            "BRACERIGHT", "BAR", "LESS", "GREATER", "QUESTION"]
  
        in_array3 = ["ASCIITILDE", "EXCLAMATION", "AT", "POUND", "DOLLAR",
            "PERCENT", "ASCIICIRCUM", "AMPERSAND", "ASTERISK", "PARENLEFT", "PARENRIGHT"]

        out_array1 = ["-", "=", ";", "'", "[", "]", "\\", ",", ".", "/"]
        out_array2 = ["_", "+", ":", '"', "{", "}", "|", "<", ">", "?"]
        alpha_upper = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        digit_row = "~!@#$%^&*()"

        if "RSHIFT" in modifiers_string or "LSHIFT" in modifiers_string:
            modifiers_string += " MOD_SHIFT"

        if key_symbol in "ABCDEFGHIJKLMNOPQRSTUVWXYZ":
            if "MOD_SHIFT" in modifiers_string:
                key_symbol = key_symbol.upper()
            else:
                key_symbol = key_symbol.lower()
        elif key_symbol in ["_1", "_2", "_3", "_4", "_5", "_6", "_7", "_8", "_9", "_0"]:
            key_symbol = key_symbol[1]
            if "MOD_SHIFT" in modifiers_string:
                key_symbol = ")!@#$%^&*("[ord(key_symbol) - ord("0")]
        elif key_symbol in in_array:
            if "MOD_SHIFT" in modifiers_string:
                key_symbol = out_array2[in_array.index(key_symbol)]
            else:
                key_symbol =  out_array1[in_array.index(key_symbol)]
        elif key_symbol in in_array2:
            key_symbol = out_array2[in_array2.index(key_symbol)]
        elif key_symbol in in_array3:
            key_symbol = digit_row[in_array3.index(key_symbol)]
        elif key_symbol == "SPACE":
            key_symbol = " "
        elif key_symbol == "BACKSPACE":
            text = self.entry_label.element.text
            if text:
                text = text[0:-1]
            self.entry_label.element.text = text
        elif key_symbol == "RETURN":
            text = self.entry_label.element.text
            self.entry_label.element.text = ""
            self._create_new_entry_label_shadow()
            self.on_word_entried(text)
        elif key_symbol == "F12":
                # Cheat.
                self.stack_trace.unwind(self)
        #last chance ...
        elif key_code in big_alpha_codes:
            key_symbol = alpha_upper[big_alpha_codes.index(key_code)]

        print key_symbol

        if key_symbol and len(key_symbol) != 1:
            key_symbol = ""

        if key_symbol:
            soundex.play('key-typed.wav')
            self.entry_label.element.text += key_symbol

    def on_word_entried(self, text):
        sound_played = False

        if self.killed:
            if text == "raise" or text == "throw":
                self.back_to_live()
                sound_played = True
        else:
            for phrase_ix, phrase in enumerate(self.phrases):
                if phrase.element.text == text:
                    pl = Label(text,
                        x=phrase.element.x,
                        y=phrase.element.y,
                        font_name="Courier", font_size=32,
                        anchor_y=phrase.element.anchor_y, anchor_x=phrase.element.anchor_x,
                        color=(0, 0, 228, 255), bold=True)
                    pl.transform_anchor = phrase.transform_anchor
                    pl.rotation = phrase.rotation
                    self.add(pl, z=80)
                    self.label_shadows.append(pl)

                    sound_played = True
                    phrase.element.text = self._pick_phrase()

                    velocity_delta = 160
                    if phrase_ix == 0:
                        self.avatar.velocity = (self.avatar.velocity[0], self.avatar.velocity[1] - velocity_delta)
                    elif phrase_ix == 1:
                        self.avatar.velocity = (self.avatar.velocity[0] - velocity_delta, self.avatar.velocity[1])
                    elif phrase_ix == 2:
                        self.avatar.velocity = (self.avatar.velocity[0] + velocity_delta, self.avatar.velocity[1])
                    elif phrase_ix == 3:
                        self.avatar.velocity = (self.avatar.velocity[0], self.avatar.velocity[1] + velocity_delta)

        if not sound_played:
            pass

        #####
        #self.stack_trace.unwind()
        #####
        soundex.play('phrase-entered.wav')

    def _step(self, dt):
        super(BattleLayer, self)._step(dt)
        self.update(dt)
        self.resume()

# -----------------------------------------------------------------------------
class BattleScene(Scene):
    def __init__(self, level_index):
        super(BattleScene, self).__init__()
        self.bs = BattleLayer(level_index)
        self.add(self.bs)

    def on_enter(self):
        super(BattleScene, self).on_enter()

    def on_exit(self):
        print "BattleScene.on_exit()"
        self.remove(self.bs)
        for a in self.get_children():
            self.remove(a)
        super(BattleScene, self).on_exit()

def get_battle_scene(level_index=0):
    scene = BattleScene(level_index)
    return scene
