import sys
import os.path

from cocos.director import director
from cocos.layer import Layer, ColorLayer, MultiplexLayer
from cocos.scene import Scene
from cocos.scenes.transitions import *
from cocos.sprite import Sprite
from cocos.menu import Menu, MenuItem, ToggleMenuItem, MultipleMenuItem, shake, shake_back, zoom_in, zoom_out
from cocos.text import Label
from cocos.actions.base_actions import *
from cocos.actions.interval_actions import *
from cocos.actions.instant_actions import *
from cocos.actions.interval_actions import Reverse, Repeat

import pyglet
from pyglet import gl, font
from pyglet.window import key

import world_scene

import soundex
from constants import *


STEPS_INTRO = ( (0, 50), (0, 50), (0, 50), (0, 50), (0, 50), (0, 50), (0, 50), (0, 50), (0, 50) )

TIME_STEP = 1.0

class BackgroundLayer( Layer ):
    is_event_handler = True
    posx = 0

    def put_common(self):
        sc = 1.8
        xpos = 500
        ypos = -100
        self.part0 = Sprite("intro_00.png", position=(xpos, ypos), scale=sc)
        self.part1 = Sprite("intro_01.png", position=(xpos, ypos), scale=sc)
        self.part2 = Sprite("intro_02.png", position=(xpos, ypos), scale=sc)
        self.part3 = Sprite("intro_03.png", position=(xpos, ypos), scale=sc)
        self.part4 = Sprite("intro_04.png", position=(xpos, ypos), scale=sc)
        self.part5 = Sprite("intro_05.png", position=(xpos, ypos), scale=sc)
        self.part6 = Sprite("intro_06.png", position=(xpos, ypos), scale=sc)
        self.part7 = Sprite("intro_07.png", position=(xpos, ypos), scale=sc)

        self.dots_layer = Layer()
        self.add(self.dots_layer, z=2)

        self.add(self.part0, z=3)
        self.add(self.part1, z=3)
        self.add(self.part2, z=3)
        self.add(self.part3, z=3)
        self.add(self.part4, z=3)
        self.add(self.part5, z=3)
        self.add(self.part6, z=3)
        self.add(self.part7, z=3)

    def play_intro(self):
        #urrent_time = start_time
        index = 0
        time1 = 23

        move1 = MoveBy( (0, 800), time1 )
        scale1 = ScaleTo( 0.5 , time1 )
        delay1 = Delay(10)
        fo1 = FadeOut(7)
        self.part0.do( scale1 )
        self.part0.do( move1 );
        self.part0.do( delay1 + fo1 )

        ff = 2.3
        delay_2 = Delay(1*ff)
        delay_3 = Delay(2*ff)
        delay_4 = Delay(3*ff)
        delay_5 = Delay(4*ff)
        delay_6 = Delay(5*ff)
        delay_7 = Delay(6*ff)
        delay_8 = Delay(7*ff)

        self.part1.do( delay_2 + scale1 )
        self.part1.do( delay_2 + move1 );
        self.part1.do( delay_2 + delay1 + fo1 )

        self.part2.do( delay_3 + scale1 )
        self.part2.do( delay_3 + move1 );
        self.part2.do( delay_3 + delay1 + fo1 )

        self.part3.do( delay_4 + scale1 )
        self.part3.do( delay_4 + move1 );
        self.part3.do( delay_4 + delay1 + fo1 )

        self.part4.do( delay_5 + scale1 )
        self.part4.do( delay_5 + move1 );
        self.part4.do( delay_5 + delay1 + fo1 )

        self.part5.do( delay_6 + scale1 )
        self.part5.do( delay_6 + move1 );
        self.part5.do( delay_6 + delay1 + fo1 )

        self.part6.do( delay_7 + scale1 )
        self.part6.do( delay_7 + move1 );
        self.part6.do( delay_7 + delay1 + fo1 )

        self.part7.do( delay_8 + scale1 )
        self.part7.do( delay_8 + move1 );
        self.part7.do( delay_8 + delay1 + fo1 )

        self.do(Delay(30) + CallFunc(soundex.stop_music))

        self.part7.do(Delay(35) + CallFunc(self._enter_map))

    def _enter_map(self):
        director.replace(world_scene.get_world_scene())

    def __init__(self):
        super( BackgroundLayer, self ).__init__()
        self.put_common()

    def on_key_press(self, key_code, modifiers):
        self._enter_map()

class IntroScene(Scene):
    def __init__(self):
        super(IntroScene, self).__init__()
        self.bkg_layer = BackgroundLayer()
        self.add(self.bkg_layer, z=1)

    def on_enter(self):
        super(IntroScene, self).on_enter()
        self.bkg_layer.play_intro()
        soundex.set_music('recitation.wav')

    def on_exit(self):
        super(IntroScene, self).on_exit()
        director.replace(world_scene.get_world_scene())
        soundex.stop_music()
        soundex.set_music('Marloc-YouAreSunshineOfMyStacktrace.mp3')
        
def get_intro_scene():
    scene = IntroScene()
    return scene

