import sys
import os.path

from cocos.director import director
from cocos.layer import Layer
from cocos.scene import Scene
from cocos.scenes.transitions import *
from cocos.text import Label
from cocos.sprite import Sprite
from cocos.actions.base_actions import *
from cocos.actions.interval_actions import *
from cocos.actions.instant_actions import *
from cocos.actions.interval_actions import Reverse, Repeat

import pyglet
from pyglet import gl, font
from pyglet.window import key

import soundex
from constants import *

import endgame_scene
import battle_scene

STEPS_DESK = ( (294, 541), (342, 519), (371, 477), (375, 416), (381, 357) )
STEPS_SERV = ( (414, 356), (424, 402), (437, 443), (494, 456), (553, 472), (588, 522), (629, 570), (666, 573), (716, 573), (752, 543) )
STEPS_SUPE = ( (745, 471), (719, 440), (693, 400), (682, 369), (656, 326), (642, 288), (642, 248), (660, 221), (705, 205), (744, 206), (773, 216), (803, 230) )

TIME_STEP = 1.0

def log(txt):
    print txt

LBL_ARGS = dict(font_name=FONTS, font_size=20, anchor_y='top', anchor_x='left', color=FONT_COLOR)

class BackgroundLayer(Layer):
    is_event_handler = True
    def put_common(self):
        self.map = Sprite("middle-earth-map.png", position=(WIDTH/2,HEIGHT/2))
        self.phone = Sprite("idroid.png", position=(300,600), scale=0.1)
        self.desktop = Sprite("desktop-computer-OK.png", position=(400,300))
        self.server = Sprite("root-nameserver.png", position=(800,550))
        self.internet = Sprite("world-internet.png", position=(900,650))
        self.cray = Sprite("world-pb-rig.png", position=(900,250))
        self.exc = Sprite("exception-avatar.png", position=STEPS_DESK[0], scale=0.5)

        self.desktop_done = Sprite("desktop-computer-blue.png", position=(400,300))
        self.desktop_done.opacity = 0

        self.sauron = Sprite("sauron-eye.png", position=(876,420), scale=0.7)
        self.phone_done = Sprite("idroid-bricked.png", position=(300,600), scale=0.1)
        self.phone_done.opacity = 0
        self.dots_layer = Layer()
        self.add(self.dots_layer, z=2)

        self.spil = []
        self.spil.append( Sprite("world-spil-0.png", position=(WIDTH/2,HEIGHT/2), opacity=0) )
        self.spil.append( Sprite("world-spil-1.png", position=(WIDTH/2,HEIGHT/2), opacity=0) )
        self.spil.append( Sprite("world-spil-2.png", position=(WIDTH/2,HEIGHT/2), opacity=0) )
        self.spil.append( Sprite("world-spil-3.png", position=(WIDTH/2,HEIGHT/2), opacity=0) )

        self.add(self.map)
        self.add(self.desktop, z=2)
        self.add(self.phone, z=2)
        self.add(self.server, z=2)
        self.add(self.cray, z=2)
        self.add(self.internet, z=1)
        self.add(self.sauron, z=3)
        self.add(self.phone_done, z=3)
        self.add(self.desktop_done, z=3)
        self.add(self.exc, z=4)

        self.game_title_lbl = Label("Exception Uncaught",
            position=(40, 130), bold=True, font_size=42, font_name=FONTS,
            anchor_y='top', anchor_x='left', color=(120, 30, 10, 230))
        self.game_title2_lbl = Label("Exception Uncaught",
            position=(42, 128), bold=True, font_size=42, font_name=FONTS,
            anchor_y='top', anchor_x='left', color=(30, 10, 0, 255))
        self.dots_layer.add(self.game_title_lbl, z=100)
        self.dots_layer.add(self.game_title2_lbl, z=80)
        self.press_enter_lbl = Label("Press Enter to start level",
            position=(40, 60), bold=True, font_size=24, font_name=FONTS,
            anchor_y='top', anchor_x='left', color=(100, 100, 20, 200))
        self.press_enter_lbl.do(Repeat(FadeIn(1) + FadeOut(1)))
        self.dots_layer.add(self.press_enter_lbl,z=10)

    def add_steps(self, layer, steps, start_time, time_step, instant):
        current_time = start_time
        exc_action = Delay(0)
        for step_pos in steps:
            s = Sprite("world-step.png", position=step_pos, scale=0.2)
            if not instant:
                s.opacity = 0
                s.do( Delay(current_time) + FadeIn(0.1) )
                exc_action = exc_action + MoveTo(step_pos, 1)
            layer.add(s)
            current_time = current_time + time_step
        self.exc.do( exc_action )
        return current_time

    def __init__(self, scene):
        super( BackgroundLayer, self ).__init__()
        self.scene = scene
        self.put_common()
        soundex.load('sfx-clunk.mp3')


    #return level to run next
    def show_stage(self, just_finished):
        curr = 0
        if just_finished < 1:
            return 1
        #Cell phone breaks.
        if just_finished == 1:
            self.phone_done.do( FadeIn(0.5) )
            self.exc.position = STEPS_DESK[0]
            curr = self.add_steps(self.dots_layer, STEPS_DESK, 1.5, TIME_STEP, False)
        else:
            self.phone_done.opacity = 255
            self.add_steps(self.dots_layer, STEPS_DESK, 1.5, TIME_STEP, True)

        if just_finished < 2:
            self.dots_layer.add(self.press_enter_lbl)
            self.press_enter_lbl.do(Delay(curr) + Show())
            return 2

        #Desktop breaks.
        if just_finished == 2:
            self.desktop_done.do( Delay(curr) + FadeIn(0.5) )
            self.exc.position = STEPS_SERV[0]
            curr = self.add_steps(self.dots_layer, STEPS_SERV, curr+0.5, TIME_STEP, False)
        else:
            self.desktop_done.opacity = 255
            self.add_steps(self.dots_layer, STEPS_SERV, curr+0.5, TIME_STEP, True)


        if just_finished < 3:
            self.dots_layer.add(self.press_enter_lbl)
            self.press_enter_lbl.do(Delay(curr) + Show())
            return 3

        #Server breaks.
        self.internet.color = (255,0,0)
        self.internet.do( Repeat(FadeOut(0.33) + FadeIn(0.33)) )

        if just_finished == 3:
            curr = self.add_steps(self.dots_layer, STEPS_SUPE, curr, TIME_STEP, False)
            self.exc.position = STEPS_SUPE[0]
        else:
            self.add_steps(self.dots_layer, STEPS_SUPE, curr, TIME_STEP, True)

        if just_finished < 4:
            self.dots_layer.add(self.press_enter_lbl)
            self.press_enter_lbl.do(Delay(curr) + Show())
            return 4


        self.exc.position = STEPS_SUPE[11]
        #Oil rig breaks.
        spil_delay = curr

        soundex.play('sfx-clunk.mp3')
        for i in self.spil:
            i.do( Delay(spil_delay) + FadeIn(1) )
            spil_delay = spil_delay + 1
            self.add(i,z=1)

        self.dots_layer.add(self.press_enter_lbl,z=2)
        self.press_enter_lbl.do(Delay(spil_delay + 1) + Show())

    def on_mouse_press(self, x, y, buttons, xxx):
        print x, y

    def on_key_press(self, k, m):
        if k == key.ENTER:
            self.press_enter_lbl.do(Hide())
            self.scene.run_next_level()
            return True
        return False

class WorldScene(Scene):
    def __init__(self):
        super(WorldScene, self).__init__()
        self.bkg_layer = BackgroundLayer(self)
        self.add(self.bkg_layer, z=1)

        self.level_num = -1 

    def run_next_level(self):
        print self.level_num

        if self.level_num == 4:
          s = endgame_scene.get_endgame_scene()
          director.run(s)
        else:
          s = battle_scene.get_battle_scene(self.level_num)
          director.push(s)



    def on_enter(self):
        super(WorldScene, self).on_enter()
        self.level_num += 1
        log("entering game scene: level %d" % (self.level_num))
        self.bkg_layer.show_stage(self.level_num)

    def on_exit(self):
        super(WorldScene, self).on_exit()
        
def get_world_scene():
    scene = WorldScene()
    return scene

